package com.ningdatech.pmapi.ding.controller;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.digest.HMac;
import cn.hutool.crypto.digest.MD5;
import com.google.common.collect.Maps;
import com.ningdatech.irs.service.IRefreshTokenService;
import com.ningdatech.pmapi.common.util.CryptUtils;
import com.ningdatech.pmapi.common.util.HttpUtil;
import com.ningdatech.pmapi.ding.task.EmployeeBatchGetTask;
import com.ningdatech.pmapi.ding.task.GovBusinessStripsTask;
import com.ningdatech.pmapi.ding.task.OrganizationBatchGetTask;
import com.ningdatech.pmapi.organization.manage.ProcDefManage;
import com.sun.crypto.provider.HmacMD5;
import com.sun.crypto.provider.HmacMD5KeyGenerator;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Objects;

/**
 * <p>
 * PullDingInfoController
 * </p>
 *
 * @author WendyYang
 * @since 2023/6/2
 **/
@RestController
@RequestMapping("/api/v1/zwdd/pull")
@RequiredArgsConstructor
@Slf4j
public class DingInfoPullController {

    private final EmployeeBatchGetTask employeeBatchGetTask;
    private final GovBusinessStripsTask businessStripsTask;
    private final OrganizationBatchGetTask organizationBatchGetTask;

    private final ProcDefManage procDefManage;

    @Resource(name = "refreshToken")
    private IRefreshTokenService refreshTokenService;

    @Value("${irs.interface-refresh.request-token-url}")
    private String govRequestTokenUrl;
    @Value("${irs.interface-refresh.refresh-token-url}")
    private String govRefreshTokenUrl;

    @GetMapping("/employee")
    public void employeeBatchGet() {
        employeeBatchGetTask.batchGetEmployeeTask();
    }

    @GetMapping("/businessStrips")
    public void businessStripsBatchGet() {
        businessStripsTask.batchGetGovBusinessStripsTask();
    }

    @GetMapping("/organization")
    public void organizationBatchGet() {
        organizationBatchGetTask.batchGetOrganizationTask();
    }

    @GetMapping("/employeeByRegionCode/{regionCode}")
    public void employeeBatchGetByRegionCode(@PathVariable String regionCode) {
        employeeBatchGetTask.batchGetEmployeeTaskByRegionCode(regionCode);
    }

    @GetMapping("/employeeByOrdCode/{orgCode}")
    public void employeeBatchGetByOrdCode(@PathVariable String orgCode) {
        employeeBatchGetTask.batchGetEmployeeTaskByOrdCode(orgCode);
    }

    @GetMapping("/getEmployeeByCode/{empCode}")
    public void getEmployeeByCode(@PathVariable String empCode) {
        employeeBatchGetTask.getEmployeeByCode(empCode);
    }

    @GetMapping("/getBatchEmployeeByCode/{orgCode}")
    public void getBatchEmployeeByCode(@PathVariable String orgCode) {
        employeeBatchGetTask.getBatchEmployeeByCode(orgCode);
    }

    @GetMapping("/getToken")
    public String getToken() {
       return employeeBatchGetTask.getToken();
    }

    @GetMapping("/getAuthToken")
    public String getAuthToken() {
        return employeeBatchGetTask.getAuthToken();
    }

    @ApiOperation(value = "系统流程配置初始化", notes = "系统流程配置初始化")
    @GetMapping("/init")
    public String init() {
        return procDefManage.init();
    }

    @ApiOperation(value = "系统流程配置初始化按区域", notes = "系统流程配置初始化按区域")
    @GetMapping("/init/{areaCode}")
    public String initByArea(@PathVariable String areaCode) {
        return procDefManage.initByArea(areaCode);
    }

    @ApiOperation(value = "测试项目归并", notes = "测试项目归并")
    @GetMapping("/test-guibing")
    public String testGuibing() throws Exception {
        long timestamp = System.currentTimeMillis();
        String appSecret = "496f0f2a19994f76b4fd9dae087366c7";
        String appKey = "A331101453557202109017383";
        String method = "POST";
        String interfaceName = "5crv7i4727abLCv0";
        String secret = refreshTokenService.refreshToken(appKey,appSecret,govRequestTokenUrl,govRefreshTokenUrl,interfaceName,method);
        log.info("secret :{}",secret);
        String sign = MD5.create().digestHex(appKey + secret + timestamp);
        HttpComponentsClientHttpRequestFactory factory = HttpUtil.generateHttpRequestFactory();
        RestTemplate restTemplate;
        if(Objects.nonNull(factory)){
            restTemplate = new RestTemplate(factory);
        }else{
            restTemplate = new RestTemplate();
        }
        String appId = "A330000100000202105005790";
        String capCode = CryptUtils.encryptHMAC(timestamp + appId, "bcb4fc0f517fc168ee543fcf4d231f3b");
        String url = "https://interface.zjzwfw.gov.cn/gateway/api/proxy/001003001029/" +
                "dataSharing/5crv7i4727abLCv0.htm?requestTime=" + timestamp +
                "&sign=" + sign + "&appKey=" + appKey +
                "&capCode=" + capCode + "&capTime=" + timestamp +
                "&appId=" + appId;
        log.info("url :{}",url);
        ResponseEntity<String> forEntity = restTemplate.getForEntity(url, String.class);
        log.info(forEntity.getBody());
        return forEntity.getBody();
    }

    @ApiOperation(value = "测试项目详情推送", notes = "测试项目详情推送")
    @GetMapping("/test-push-project")
    public String testPushProject() throws Exception {
        long timestamp = System.currentTimeMillis();
        String appSecret = "496f0f2a19994f76b4fd9dae087366c7";
        String appKey = "A331101453557202109017383";
        String method = "POST";
        String interfaceName = "3XN9R93Pva6db7sf";
        String secret = refreshTokenService.refreshToken(appKey,appSecret,govRequestTokenUrl,govRefreshTokenUrl,interfaceName,method);
        log.info("secret :{}",secret);
        String sign = MD5.create().digestHex(appKey + secret + timestamp);
        HttpComponentsClientHttpRequestFactory factory = HttpUtil.generateHttpRequestFactory();
        RestTemplate restTemplate;
        if(Objects.nonNull(factory)){
            restTemplate = new RestTemplate(factory);
        }else{
            restTemplate = new RestTemplate();
        }
        String baseProjPrincipal = "蒋国海";
        String baseProjSetYear = "2022";
        String baseProjId = "331123220130969000000";
        String baseManDeprt = "遂昌县大数据和金融发展中心";
        String baseProjBasis = "2";
        String baseLowestLevel = "C";
        String ProjPrincipalCall = "13884366607";
        //内容类别
        String baseConstructionType = "01";
        String baseProjType = "1";
        String baseProjContacts = "毛以林";
        String isEffective = "1";
        String baseConsDeprtDing = "GO_9a7dce7fbaf54787998fa004f1b7c0e1";
        String baseProjSetProg = "04";
        String baseProjContactsCall = "13857049999";
        String baseProjName = "遂昌县大数据和金融发展中心天工系统";
        String baseProjConsClass = "C";
        String areaCode = "331123000";
        String baseAreaName = "遂昌县";
        String baseManDeprtDing = "GO_9a7dce7fbaf54787998fa004f1b7c0e1";
        String baseProvManDeprtDing = "GO_6c383c049d95461f9a0df780140ceb32";
        String baseBasisEstablish = "中共遂昌县委办公室关于研究开发“天工”智能化管理平台（三期）的函";
        String capCode = CryptUtils.encryptHMAC(timestamp + areaCode, "bcb4fc0f517fc168ee543fcf4d231f3b");
        String url = "https://interface.zjzwfw.gov.cn/gateway/api/proxy/001003001029/dataSharing/3XN9R93Pva6db7sf.htm?" +
                "requestTime=" + timestamp + "&sign=" + sign +
                "&appKey=" + appKey + "&baseProjPrincipal=" + baseProjPrincipal + "" +
                "&baseProjSetYear=" + baseProjSetYear + "&baseProjId=" + baseProjId +
                "&baseManDeprt=" + baseManDeprt + "&baseProjBasis=" + baseProjBasis +
                "&baseLowestLevel=" + baseLowestLevel + "&ProjPrincipalCall=" + ProjPrincipalCall +
                "&baseConstructionType=" + baseConstructionType + "&baseProjType=" + baseProjType +
                "&baseProjContacts=" + baseProjContacts + "&isEffective=" + isEffective +
                "&baseConsDeprtDing=" + baseConsDeprtDing + "&baseProjSetProg=" + baseProjSetProg +
                "&baseProjContactsCall=" + baseProjContactsCall + "&baseProjName=" + baseProjName +
                "&baseProjConsClass=" + baseProjConsClass + "&capCode=" + capCode + "&baseManDeprtDing=" + baseManDeprtDing +
                "&baseProvManDeprtDing=" + baseProvManDeprtDing + "&baseBasisEstablish=" + baseBasisEstablish +
                "&baseAreaCode=" + areaCode + "&capTime=" + timestamp + "&baseAreaName=" + baseAreaName;
        log.info("url :{}",url);
        ResponseEntity<String> forEntity = restTemplate.getForEntity(url, String.class);
        log.info(forEntity.getBody());
        return forEntity.getBody();
    }

    public static void main(String[] args) throws Exception {
        long timestamp = System.currentTimeMillis();
        String areaCode = "331123000";
        String res = CryptUtils.encryptHMAC(timestamp + areaCode, "bcb4fc0f517fc168ee543fcf4d231f3b");
        System.out.println(res);
    }

    /**
     * 将byte数组转为十六进制字符串
     * @param bytes byte数组
     * @return 十六进制字符串
     */
    private static String bytesToHex(byte[] bytes) {
        StringBuilder hex = new StringBuilder();
        for (byte b : bytes) {
            hex.append(String.format("%02x", b));
        }
        return hex.toString();
    }
}
